-- Lua Status widget
-- 01 alpha 9
-- 02 alpha 10 font changed
-- 03 bugfixes
-- 04 invert source for system events

local translations = {en="Lua Status"}

local function name(widget)
    local locale = system.getLocale()
    return translations[locale] or translations["en"]
end

local function create()
    return {color1=lcd.RGB(0xEA, 0x5E, 0x00), color2=lcd.RGB(0x00, 0x00, 0x00), source=nil, value=0, fontsize=FONT_STD, activetext="", inactivetext=""}
end

local function paint(widget)
    if widget.source == nil then
        return
    end

    lcd.font(widget.fontsize)

    local w, h = lcd.getWindowSize()
    local text_w, text_h = lcd.getTextSize("")
    -- local box_top, box_height = text_h, h - text_h - 4
    -- local box_left, box_width = 4, w - 8

    -- Source name and value
    -- lcd.drawText(box_left, 0, widget.source:name())
    -- lcd.drawText(box_left + box_width, 0, widget.source:stringValue(), RIGHT)

    if widget.value < 0 then
        -- Inactive background
        -- lcd.color(widget.ri, widget.gi, widget.bi)
        lcd.color(widget.color2)
        lcd.drawFilledRectangle(0, 0, w, h)
        -- Inactive Text
        lcd.color(BLACK)
        lcd.drawText(w / 2, (h - text_h)/ 2, widget.inactivetext, CENTERED)
    else
        -- Active background
        -- lcd.color(widget.r, widget.g, widget.b)
        lcd.color(widget.color1)
        lcd.drawFilledRectangle(0, 0, w, h)
        print("lcd.color"..widget.color1) -- debug
        -- Active Text
        lcd.color(BLACK)
        lcd.drawText(w / 2, (h - text_h)/ 2, widget.activetext, CENTERED)
    end

end

local function wakeup(widget)
    if widget.source then
        local newValue = widget.source:value()
        if widget.value ~= newValue then
            -- print("lcd.invalidate "..widget.value.." "..newValue)
            widget.value = newValue
            lcd.invalidate()
end
    end
end

local function configure(widget)
    -- Source choice
    line = form.addLine("Source")
    form.addSourceField(line, nil, function() return widget.source end, function(value) widget.source = value end)

    -- Active text param
    line = form.addLine("Active Text Parameter")
    form.addTextField(line, nil, function() return widget.activetext end, function(value) widget.activetext = value end);
    
    -- Active Color
    line = form.addLine("Active Background Color")
    --form.addColorField(line, nil, function() return widget.r, widget.g, widget.b end, function(r, g, b) widget.r, widget.g, widget.b = r, g, b end)
    form.addColorField(line, nil, function() return widget.color1 end, function(color1) widget.color1 = color1 end)
    print("addColorField"..widget.color1) -- debug

    -- Inactive text param
    line = form.addLine("Inactive Text Parameter")
    form.addTextField(line, nil, function() return widget.inactivetext end, function(value) widget.inactivetext = value end);
    
    -- Inactive Color
    line = form.addLine("Inactive Background Color")
    -- form.addColorField(line, nil, function() return widget.ri, widget.gi, widget.bi end, function(ri, gi, bi) widget.ri, widget.gi, widget.bi = ri, gi, bi end)
    form.addColorField(line, nil, function() return widget.color2 end, function(color2) widget.color2 = color2 end)

    -- Font (XS, S, STD, L, L Bold, XL, XXL)
    line = form.addLine("Font Size")
    form.addChoiceField(line, nil, {{"Extra Small", FONT_XS}, {"Small", FONT_S}, {"Standard", FONT_STD}, {"Large", FONT_L}, {"Large Bold", FONT_L_BOLD}, {"Extra Large", FONT_XL}, {"Extra Extra Large", FONT_XXL}}, function() return widget.fontsize end, function(value) widget.fontsize = value end)
end

local function read(widget)
    widget.source = storage.read("source")
    widget.activetext = storage.read("activetext")
    widget.inactivetext = storage.read("inactivetext")
    widget.fontsize = storage.read("fontsize")
    widget.color1 = storage.read("color1")
    widget.color2 = storage.read("color2")
end

local function write(widget)
    storage.write("source", widget.source)
    storage.write("activetext", widget.activetext)
    storage.write("inactivetext", widget.inactivetext)
    storage.write("fontsize", widget.fontsize)
    storage.write("color1", widget.color1)
    storage.write("color2", widget.color2)
end

local function init()
    system.registerWidget({key="status", name=name, create=create, paint=paint, wakeup=wakeup, configure=configure, read=read, write=write})
end

return {init=init}